/* eslint-disable @typescript-eslint/no-explicit-any */
import assert from 'assert';
import { initKmoreEvent } from './config.js';
import { processJoinTableColumnAlias } from './smart-join.js';
export function callCbOnStart(options) {
    const queryBuilder = processJoinTableColumnAlias(options.builder);
    const cb = options.cbs?.start;
    if (typeof cb === 'function') {
        const event = processKnexOnEvent({
            type: 'start',
            data: void 0,
            queryUid: '',
            queryBuilder,
            kmoreQueryId: options.kmoreQueryId,
        });
        event.dbId = options.dbId;
        cb(event, options.ctx);
        return;
    }
}
export function callCbOnQuery(options) {
    const cb = options.cbs?.query;
    if (typeof cb === 'function') {
        const queryUid = pickQueryUidFrom(options.data);
        const event = processKnexOnEvent({
            type: 'query',
            data: options.data,
            queryUid,
            queryBuilder: void 0,
            kmoreQueryId: options.kmoreQueryId,
        });
        event.dbId = options.dbId;
        cb(event, options.ctx);
        return;
    }
}
export function callCbOnQueryResp(options) {
    const cb = options.cbs?.queryResponse;
    if (typeof cb === 'function') {
        const queryUid = pickQueryUidFrom(options.respRaw);
        const event = processKnexOnEvent({
            type: 'queryResponse',
            respRaw: options.respRaw,
            queryUid,
            queryBuilder: void 0,
            kmoreQueryId: options.kmoreQueryId,
        });
        event.dbId = options.dbId;
        cb(event, options.ctx);
        return;
    }
}
export function callCbOnQueryError(options) {
    const cb = options.cbs?.queryError;
    if (typeof cb === 'function') {
        const queryUid = pickQueryUidFrom(options.data);
        const event = processKnexOnEvent({
            type: 'queryError',
            exError: options.err,
            exData: options.data,
            queryUid,
            queryBuilder: void 0,
            kmoreQueryId: options.kmoreQueryId,
        });
        event.dbId = options.dbId;
        return cb(event, options.ctx);
    }
}
function processKnexOnEvent(input) {
    const ev = {
        ...initKmoreEvent,
        ...input,
        timestamp: Date.now(),
    };
    assert(ev.kmoreQueryId, 'kmoreQueryId is required');
    switch (ev.type) {
        case 'start': {
            break;
        }
        case 'query': {
            const { data } = input;
            assert(data, 'data should be set on query event');
            ev.method = data.method ? data.method : '';
            ev.kUid = data.__knexUid;
            ev.trxId = data.__knexTxId ? data.__knexTxId : void 0;
            break;
        }
        case 'queryResponse': {
            const data = input.respRaw;
            assert(data, 'data should be set on queryResponse event');
            ev.method = data.method ? data.method : '';
            ev.command = data.response?.command;
            ev.kUid = data.__knexUid;
            ev.trxId = data.__knexTxId ? data.__knexTxId : void 0;
            break;
        }
        case 'queryError': {
            const data = input.exData;
            assert(data, 'data should be set on queryError event');
            ev.method = data.method;
            ev.kUid = data.__knexUid;
            ev.trxId = data.__knexTxId ? data.__knexTxId : void 0;
            break;
        }
        default:
            break;
    }
    return ev;
}
function pickQueryUidFrom(input) {
    return input.__knexQueryUid ? input.__knexQueryUid : '';
}
//# sourceMappingURL=event.js.map